import os
import json
import unittest
import hh_gen_toy as hh

class ToyPopulationTest(unittest.TestCase):
    def setUp(self):
        self.is_debugging = False
        self.household_dict = None
        self.household_filename = None

    def tearDown(self):
        if self.household_filename:
            os.unlink(self.household_filename)

    def gen_households(self, population_size):
        self.household_dict = hh.build_households(population_size)

    def write_household_file(self, population_size):
        self.household_filename = hh.write_households(population_size)
        with open (self.household_filename) as infile:
            self.household_dict = json.load(infile)

    def test_produces_dict(self):
        self.assertIsNone(self.household_dict)
        self.gen_households(50)
        self.assertIsNotNone(self.household_dict)
        self.assertIsInstance(self.household_dict, dict,
                              msg=f"Calling build_households() should"
                                  f" produce a dict. Got: {type(self.household_dict)}")


    def test_writes_a_file(self):
        people_to_make = 50
        expected_filename = f"households_{people_to_make}.json"
        self.assertIsNone(self.household_filename)
        self.write_household_file(50)
        self.assertIsNotNone(self.household_filename)
        self.assertEqual(self.household_filename, expected_filename,
                         msg=f"Expected filename: {expected_filename}"
                             f" Got filename: {self.household_filename}")
        with open(self.household_filename) as infile:
            tmp_households = json.load(infile)
        self.assertEqual(tmp_households, self.household_dict,
                         "This should _probably_ never fail. Making sure"
                         " that the file is json loadable.")

    def test_popsize_honored(self):
        for popsize in [50, 100, 1000, 2000, 10000]:
            self.gen_households(popsize)
            actual_size = 0
            for k in self.household_dict:
                actual_size += len(self.household_dict[k])
            self.assertGreaterEqual(actual_size, popsize,
                                    msg=f"Expected at least {popsize} people, got {actual_size}")
            self.assertLessEqual(actual_size, popsize + 10,
                                 msg=f"Expected right around {popsize} people, got {actual_size}")

    def test_pop_ratios(self):
        for popsize in [5000, 10000, 20000, 40000, 80000]:
            self.gen_households(popsize)
            work = 0
            community = 0
            highschool = 0
            juniorhigh = 0
            elementary = 0
            mystery = []
            for k in self.household_dict:
                house = self.household_dict[k]
                for person in house:
                    if person == 'W':
                        work += 1
                    elif person == "C":
                        community += 1
                    elif person == "HS":
                        highschool += 1
                    elif person == "JH":
                        juniorhigh += 1
                    elif person == "ES":
                        elementary += 1
                    else:
                        mystery.append(person)
            # Make sure there are more "work" people than anything else
            self.assertGreater(work, community)
            self.assertGreater(work, highschool)
            self.assertGreater(work, juniorhigh)
            self.assertGreater(work, elementary)

            # Make sure that each of the school groups is approximately equal
            hs_frac = highschool / popsize
            jh_frac = juniorhigh / popsize
            es_frac = elementary / popsize
            if self.is_debugging:
                print(f"At popsize {popsize}, hs_frac: {hs_frac} jh_frac: {jh_frac} es_frac: {es_frac}")
            for fraction in [hs_frac, jh_frac, es_frac]:
                self.assertGreater(fraction, 0.12)
                self.assertLess(fraction, 0.16)
                # NOTE: This is just back of the envelope


    def test_all_households_have_worker(self):
        self.gen_households(1000)
        for h in self.household_dict:
            self.assertIn("W", self.household_dict[h]) 
